==> Wikibase-REL1_35-56582b4.tar.gz: Wikibase/client/tests/phpunit/integration/includes/Usage/UsageTrackingIntegrationTest.php <==
<?php

namespace Wikibase\Client\Tests\Integration\Usage;

use MediaWiki\MediaWikiServices;
use MediaWikiTestCase;
use Title;
use Wikibase\Client\Usage\EntityUsage;
use Wikibase\Client\WikibaseClient;
use Wikibase\DataModel\Entity\ItemId;
use Wikibase\DataModel\Entity\Property;
use Wikibase\Lib\WikibaseSettings;
use Wikibase\Repo\WikibaseRepo;
use WikiPage;
use WikitextContent;

/**
 * @group Wikibase
 * @group WikibaseClient
 * @group WikibaseUsageTracking
 *
 * @group Database
 *
 * @license GPL-2.0-or-later
 * @author Daniel Kinzler
 */
class UsageTrackingIntegrationTest extends MediaWikiTestCase {

	/**
	 * @var Title
	 */
	private $articleTitle;

	/**
	 * @var Title
	 */
	private $templateTitle;

	/**
	 * @var bool
	 */
	private $oldAllowDataTransclusion;

	/**
	 * @var int[]
	 */
	private $oldEntityNamespaces;

	protected function setUp(): void {
		if ( !WikibaseSettings::isRepoEnabled() ) {
			$this->markTestSkipped( 'Integration test requires repo and client extension to be active on the same wiki.' );
		}

		parent::setUp();

		$wikibaseClient = WikibaseClient::getDefaultInstance();

		$settings = $wikibaseClient->getSettings();
		$this->oldAllowDataTransclusion = $settings->getSetting( 'allowDataTransclusion' );
		$this->oldEntityNamespaces = $wikibaseClient->getEntityNamespaceLookup()->getEntityNamespaces();
		$settings->setSetting( 'allowDataTransclusion', true );
		$settings->setSetting( 'entityNamespaces', [ 'item' => 0 ] );

		$ns = $this->getDefaultWikitextNS();
		$this->articleTitle = Title::makeTitle( $ns, 'UsageTrackingIntegrationTest_Article' );
		$this->templateTitle = Title::makeTitle( NS_TEMPLATE, 'UsageTrackingIntegrationTest_Template' );
	}

	protected function tearDown(): void {
		parent::tearDown();

		WikibaseClient::getDefaultInstance()->getSettings()->setSetting(
			'allowDataTransclusion',
			$this->oldAllowDataTransclusion
		);
		WikibaseClient::getDefaultInstance()->getSettings()->setSetting(
			'entityNamespaces',
			$this->oldEntityNamespaces
		);
	}

	private function runJobs() {
		MediaWikiServices::getInstance()->getJobRunner()->run( [
			'type'     => 'refreshLinks',
			'maxJobs'  => false,
			'maxTime'  => false,
			'throttle' => false,
		] );
	}

	private function deletePage( Title $title ) {
		$page = WikiPage::factory( $title );
		$page->doDeleteArticleReal( 'TEST', $this->getTestSysop()->getUser() );

		$this->runJobs();

		$title->resetArticleID( false );
	}

	private function updatePage( Title $title, $text ) {
		$content = new WikitextContent( $text );

		$flags = $title->exists() ? EDIT_UPDATE : EDIT_NEW;

		$page = WikiPage::factory( $title );
		$page->doEditContent( $content, 'TEST', $flags );

		$this->runJobs();

		$title->resetArticleID( false );
	}

	public function testUpdateUsageOnCreation() {
		// Create property
		$propertyIdString = $this->storeNewProperty()->getSerialization();

		// Create a new page that uses Q11.
		$text = "Just some text\n";
		$text .= "using a property: {{#property:$propertyIdString|from=Q11}}\n";
		$this->updatePage( $this->articleTitle, $text );

		// Check that the usage of Q11 is tracked.
		$expected = [
			new EntityUsage( new ItemId( 'Q11' ), EntityUsage::STATEMENT_USAGE, $propertyIdString ),
		];

		$this->assertTrackedUsages( $expected, $this->articleTitle );
	}

	private function waitForNextTimestamp() {
		$timestamp = wfTimestampNow();

		do {
			usleep( 100 * 1000 );
		} while ( wfTimestampNow() === $timestamp );
	}

	/**
	 * @depends testUpdateUsageOnCreation
	 */
	public function testUpdateUsageOnEdit() {
		$this->waitForNextTimestamp(); // make sure we don't get the same timestamp as the edit before!
		// Create property
		$propertyIdString = $this->storeNewProperty()->getSerialization();

		// Create the template we'll use below.
		$text = "{{#property:$propertyIdString|from=Q22}}\n";
		$this->updatePage( $this->templateTitle, $text );

		// Assume the state created by testUpdateUsageOnCreation().
		// Change page content to use the template instead of {{#property}} directly.
		$text = "Just some text\n";
		$text .= "using a template: {{" . $this->templateTitle->getPrefixedText() . "}}\n";
		$this->updatePage( $this->articleTitle, $text );

		// Check that Q22, used via the template, is now tracked.
		// Check that Q11 is no longer tracked, due to timestamp-based pruning.
		$expected = [
			new EntityUsage( new ItemId( 'Q22' ), EntityUsage::STATEMENT_USAGE, $propertyIdString ),
		];

		$this->assertTrackedUsages( $expected, $this->articleTitle );
	}

	/**
	 * @depends testUpdateUsageOnEdit
	 */
	public function testUpdateUsageOnTemplateChange() {
		$this->waitForNextTimestamp(); // Make sure we don't get the same timestamp as the edit before!
		// Create property
		$propertyIdString = $this->storeNewProperty()->getSerialization();

		// Assume the state created by testUpdateUsageOnEdit().
		// Change the template to use Q33.
		$text = "{{#property:$propertyIdString|from=Q33}}\n";
		$this->updatePage( $this->templateTitle, $text );

		// Check that Q33, now used via the template, is tracked.
		// Check that Q22 is no longer tracked, due to timestamp-based pruning.
		$expected = [
			new EntityUsage( new ItemId( 'Q33' ), EntityUsage::STATEMENT_USAGE, $propertyIdString ),
		];

		$this->assertTrackedUsages( $expected, $this->articleTitle );
	}

	/**
	 * @depends testUpdateUsageOnTemplateChange
	 */
	public function testUpdateUsageOnDelete() {
		$this->waitForNextTimestamp(); // make sure we don't get the same timestamp as the edit before!

		// Assume the state created by testUpdateUsageOnTemplateChange().
		// Delete the page.
		$this->deletePage( $this->articleTitle );

		// Make sure tracking has been removed for all usages on the deleted page.
		$this->assertTrackedUsages( [], $this->articleTitle );
	}

	protected function storeNewProperty() {
		$store = WikibaseRepo::getDefaultInstance()->getEntityStore();

		$property = Property::newFromType( 'string' );
		$entityRevision = $store->saveEntity( $property, 'testing', $this->getTestUser()->getUser(), EDIT_NEW );
		return $entityRevision->getEntity()->getId();
	}

	/**
	 * @param EntityUsage[] $expected
	 * @param Title $title
	 * @param string $msg
	 */
	private function assertTrackedUsages( array $expected, Title $title, $msg = '' ) {
		$lookup = WikibaseClient::getDefaultInstance()->getStore()->getUsageLookup();
		$actual = $lookup->getUsagesForPage( $title->getArticleID() );

		$expectedUsageStrings = $this->getUsageStrings( $expected );
		$actualUsageStrings = $this->getUsageStrings( $actual );

		$this->assertEquals( $expectedUsageStrings, $actualUsageStrings, $msg );
	}

	/**
	 * @param EntityUsage[] $usages
	 *
	 * @return string[]
	 */
	private function getUsageStrings( array $usages ) {
		$strings = array_map( function ( EntityUsage $usage ) {
			return $usage->getIdentityString();
		}, $usages );

		sort( $strings );
		return $strings;
	}

}

==> Wikibase-REL1_38-132412b.tar.gz: Wikibase/client/tests/phpunit/integration/includes/Usage/UsageTrackingIntegrationTest.php <==
<?php

namespace Wikibase\Client\Tests\Integration\Usage;

use MediaWikiIntegrationTestCase;
use Title;
use Wikibase\Client\Usage\EntityUsage;
use Wikibase\Client\WikibaseClient;
use Wikibase\DataModel\Entity\EntityId;
use Wikibase\DataModel\Entity\ItemId;
use Wikibase\DataModel\Entity\NumericPropertyId;
use Wikibase\DataModel\Services\Lookup\EntityLookup;
use Wikibase\Lib\WikibaseSettings;
use WikitextContent;

/**
 * @group Wikibase
 * @group WikibaseClient
 * @group WikibaseUsageTracking
 *
 * @group Database
 *
 * @license GPL-2.0-or-later
 * @author Daniel Kinzler
 */
class UsageTrackingIntegrationTest extends MediaWikiIntegrationTestCase {

	/**
	 * @var Title
	 */
	private $articleTitle;

	/**
	 * @var Title
	 */
	private $templateTitle;

	/**
	 * @var bool
	 */
	private $oldAllowDataTransclusion;

	protected function setUp(): void {
		if ( !WikibaseSettings::isRepoEnabled() ) {
			$this->markTestSkipped( 'Integration test requires repo and client extension to be active on the same wiki.' );
		}

		parent::setUp();
		$this->tablesUsed[] = 'page';

		$settings = WikibaseClient::getSettings();
		$this->oldAllowDataTransclusion = $settings->getSetting( 'allowDataTransclusion' );
		$settings->setSetting( 'allowDataTransclusion', true );

		$entityLookup = $this->createMock( EntityLookup::class );
		$entityLookup->method( 'hasEntity' )
			->willReturnCallback( static function ( EntityId $id ) {
				// statement usage is only tracked for existing properties,
				// so pretend all properties exist
				return $id instanceof NumericPropertyId;
			} );
		$this->setService( 'WikibaseClient.EntityLookup', $entityLookup );

		$ns = $this->getDefaultWikitextNS();
		$this->articleTitle = Title::makeTitle( $ns, 'UsageTrackingIntegrationTest_Article' );
		$this->templateTitle = Title::makeTitle( NS_TEMPLATE, 'UsageTrackingIntegrationTest_Template' );
	}

	protected function tearDown(): void {
		parent::tearDown();

		WikibaseClient::getSettings()->setSetting(
			'allowDataTransclusion',
			$this->oldAllowDataTransclusion
		);
	}

	private function runRefreshLinksJobs() {
		$this->runJobs( [ 'minJobs' => 0 ], [ 'type' => 'refreshLinks' ] );
	}

	private function deletePageWithPostUpdates( Title $title ) {
		$page = $this->getServiceContainer()->getWikiPageFactory()->newFromTitle( $title );
		$page->doDeleteArticleReal( 'TEST', $this->getTestSysop()->getUser() );

		$this->runRefreshLinksJobs();

		$title->resetArticleID( false );
	}

	private function updatePage( Title $title, $text ) {
		$content = new WikitextContent( $text );

		$flags = $title->exists() ? EDIT_UPDATE : EDIT_NEW;

		$page = $this->getServiceContainer()->getWikiPageFactory()->newFromTitle( $title );
		$page->doUserEditContent(
			$content,
			$this->getTestUser()->getUser(),
			'TEST',
			$flags
		);

		$this->runRefreshLinksJobs();

		$title->resetArticleID( false );
	}

	public function testUpdateUsage() {
		// Create a new page that uses Q11.
		$text = "Just some text\n";
		$text .= "using a property: {{#property:P123|from=Q11}}\n";
		$this->updatePage( $this->articleTitle, $text );

		// Check that the usage of Q11 is tracked.
		$expected = [
			new EntityUsage( new ItemId( 'Q11' ), EntityUsage::STATEMENT_USAGE, 'P123' ),
		];
		$this->assertTrackedUsages( $expected, $this->articleTitle );

		// Create the template we'll use below.
		$text = "{{#property:P123|from=Q22}}\n";
		$this->updatePage( $this->templateTitle, $text );

		// Change page content to use the template instead of {{#property}} directly.
		$text = "Just some text\n";
		$text .= "using a template: {{" . $this->templateTitle->getPrefixedText() . "}}\n";
		$this->updatePage( $this->articleTitle, $text );

		// Check that Q22, used via the template, is now tracked.
		// Check that Q11 is no longer tracked, due to timestamp-based pruning.
		$expected = [
			new EntityUsage( new ItemId( 'Q22' ), EntityUsage::STATEMENT_USAGE, 'P123' ),
		];
		$this->assertTrackedUsages( $expected, $this->articleTitle );

		// Change the template to use Q33.
		$text = "{{#property:P123|from=Q33}}\n";
		$this->updatePage( $this->templateTitle, $text );

		// Check that Q33, now used via the template, is tracked.
		// Check that Q22 is no longer tracked, due to timestamp-based pruning.
		$expected = [
			new EntityUsage( new ItemId( 'Q33' ), EntityUsage::STATEMENT_USAGE, 'P123' ),
		];
		$this->assertTrackedUsages( $expected, $this->articleTitle );

		// Delete the page.
		$this->deletePageWithPostUpdates( $this->articleTitle );

		// Make sure tracking has been removed for all usages on the deleted page.
		$this->assertTrackedUsages( [], $this->articleTitle );
	}

	/**
	 * @param EntityUsage[] $expected
	 * @param Title $title
	 * @param string $msg
	 */
	private function assertTrackedUsages( array $expected, Title $title, $msg = '' ) {
		$lookup = WikibaseClient::getStore()->getUsageLookup();
		$actual = $lookup->getUsagesForPage( $title->getArticleID() );

		$expectedUsageStrings = $this->getUsageStrings( $expected );
		$actualUsageStrings = $this->getUsageStrings( $actual );

		$this->assertEquals( $expectedUsageStrings, $actualUsageStrings, $msg );
	}

	/**
	 * @param EntityUsage[] $usages
	 *
	 * @return string[]
	 */
	private function getUsageStrings( array $usages ) {
		$strings = array_map( function ( EntityUsage $usage ) {
			return $usage->getIdentityString();
		}, $usages );

		sort( $strings );
		return $strings;
	}

}
